@@ -1,5 +1,16 @@
 #!/usr/bin/env python
 
+"""
+Provides a cross-platform way to figure out the system uptime.
+
+Should work on damned near any operating system you can realistically expect
+to be asked to write Python code for.
+If this module is invoked as a stand-alone script, it will print the current
+uptime in a human-readable format, or display an error message if it can't,
+to standard output.
+
+"""
+
 from __future__ import with_statement
 
 import ctypes
@@ -7,14 +18,6 @@ import struct
 import sys
 import time
 
-"""
-Contains a bunch of functions that implement platform-specific ways to figure
-out the system uptime, and one that uses all of them to figure out yours.
-In principle, none of these functions create new processes or raise any
-exceptions. They all return a float representing the number of seconds of
-uptime, or None if they couldn't figure it out.
-"""
-
 def _uptime_linux():
     """Returns uptime in seconds or None, on Linux."""
     # With procfs
@@ -40,7 +43,7 @@ def _uptime_linux():
             return None
 
     if not hasattr(libc, 'sysinfo'):
-        # Not Linux
+        # Not Linux.
         return None
 
     buf = ctypes.create_string_buffer(128) # 64 suffices on 32-bit, whatever.
@@ -63,22 +66,22 @@ def _uptime_bsd():
             return None
     
     if not hasattr(libc, 'sysctlbyname'):
-        # Not BSD
+        # Not BSD.
         return None
 
-    # Determine how much space we need for the response
+    # Determine how much space we need for the response.
     sz = ctypes.c_uint(0)
     libc.sysctlbyname('kern.boottime', None, ctypes.byref(sz), None, 0)
     if sz.value != struct.calcsize('@LL'):
         # Unexpected, let's give up.
         return None
 
-    # For real now
+    # For real now.
     buf = ctypes.create_string_buffer(sz.value)
     libc.sysctlbyname('kern.boottime', buf, ctypes.byref(sz), None, 0)
     sec, usec = struct.unpack('@LL', buf.raw)
 
-    # OS X disagrees what that second value is
+    # OS X disagrees what that second value is.
     if usec > 1000000:
         usec = 0.
 
@@ -92,7 +95,7 @@ def _uptime_plan9():
     """Returns uptime in seconds or None, on Plan 9."""
     # Apparently Plan 9 only has Python 2.2, which I'm not prepared to
     # support. Maybe some Linuxes implement /dev/time, though, someone was
-    # taling about it somewhere.
+    # talking about it somewhere.
     try:
         with open('/dev/time', 'r') as f:
             # The time file holds one 32-bit number representing the sec-
@@ -106,7 +109,7 @@ def _uptime_plan9():
         return None
 
 def _uptime_solaris():
-    """Returns uptime in second or None, on Solaris."""
+    """Returns uptime in seconds or None, on Solaris."""
     try:
         kstat = ctypes.CDLL('libkstat.so')
     except OSError:
@@ -116,41 +119,12 @@ def _uptime_solaris():
     boottime = None
 
     # Unfortunately, getting at it isn't perfectly straightforward.
-    # But we're going to do this in Python if it kills us.
-
     # First, let's pretend to be kstat.h
 
     # Constant
     KSTAT_STRLEN = 31   # According to every kstat.h I could find.
 
-    # Structs
-    class kstat_t(ctypes.Structure):
-        # One of kstat_t's fields is a pointer to kstat_t, so...
-        pass
-    kstat_t._fields_ = [('ks_crtime', ctypes.c_uint64),
-                        ('ks_next', ctypes.POINTER(kstat_t)),
-                        ('ks_kid', ctypes.c_int),
-                        ('ks_module', ctypes.c_char * KSTAT_STRLEN),
-                        ('ks_resv', ctypes.c_char),
-                        ('ks_instance', ctypes.c_int),
-                        ('ks_name', ctypes.c_char * KSTAT_STRLEN),
-                        ('ks_type', ctypes.c_char),
-                        ('ks_class', ctypes.c_char * KSTAT_STRLEN),
-                        ('ks_flags', ctypes.c_char),
-                        ('ks_data', ctypes.c_voidp),
-                        ('ks_ndata', ctypes.c_uint),
-                        ('ks_data_size', ctypes.c_uint),
-                        ('ks_snaptime', ctypes.c_uint64),
-                        ('ks_update', ctypes.c_void_p),
-                        ('ks_private', ctypes.c_void_p),
-                        ('ks_snapshot', ctypes.c_void_p),
-                        ('ks_lock', ctypes.c_void_p)]
-
-    class kstat_ctl_t(ctypes.Structure):
-        _fields_ = [('kc_chain_id', ctypes.c_int),
-                    ('kc_chain', ctypes.POINTER(kstat_t)),
-                    ('kc_kd', ctypes.c_int)]
-
+    # Data structures
     class anon_union(ctypes.Union):
         # The ``value'' union in kstat_named_t actually has a bunch more
         # members, but we're only using it for boot_time, so we only need
@@ -164,18 +138,18 @@ def _uptime_solaris():
                     ('value', anon_union)]
 
     # Function signatures
-    kstat.kstat_open.restype = ctypes.POINTER(kstat_ctl_t)
-    kstat.kstat_lookup.restype = ctypes.POINTER(kstat_t)
-    kstat.kstat_lookup.argtypes = [ctypes.POINTER(kstat_ctl_t),
+    kstat.kstat_open.restype = ctypes.c_void_p
+    kstat.kstat_lookup.restype = ctypes.c_void_p
+    kstat.kstat_lookup.argtypes = [ctypes.c_void_p,
                                    ctypes.c_char_p,
                                    ctypes.c_int,
                                    ctypes.c_char_p]
     kstat.kstat_read.restype = ctypes.c_int
-    kstat.kstat_read.argtypes = [ctypes.POINTER(kstat_ctl_t),
-                                 ctypes.POINTER(kstat_t),
+    kstat.kstat_read.argtypes = [ctypes.c_void_p,
+                                 ctypes.c_void_p,
                                  ctypes.c_void_p]
     kstat.kstat_data_lookup.restype = ctypes.POINTER(kstat_named_t)
-    kstat.kstat_data_lookup.argtypes = [ctypes.POINTER(kstat_t),
+    kstat.kstat_data_lookup.argtypes = [ctypes.c_void_p,
                                         ctypes.c_char_p]
 
     # Now, let's do something useful.
@@ -193,7 +167,6 @@ def _uptime_solaris():
             boottime = data.contents.value.time
 
     # Clean-up.
-    # FIXME I don't know if this also frees the buffer allocated by kstat_read.
     kstat.kstat_close(kc)
 
     if boottime is not None:
@@ -213,10 +186,10 @@ def _uptime_windows():
     if not hasattr(ctypes, 'windll') or not hasattr(ctypes.windll, 'kernel32'):
         return None
     if hasattr(ctypes.windll.kernel32, 'GetTickCount64'):
-        # Vista/Server 2008 or later
+        # Vista/Server 2008 or later.
         return ctypes.windll.kernel32.GetTickCount64() / 1000.
     if hasattr(ctypes.windll.kernel32, 'GetTickCount'):
-        # Win2k or later; gives wrong answers after 49.7 days
+        # Win2k or later; gives wrong answers after 49.7 days.
         return ctypes.windll.kernel32.GetTickCount() / 1000.
     return None
 
